﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace HostingApplication
{
    public partial class HostControl : Control
    {
        private string _executablePath;
        private IntPtr _hostedAppHandle = IntPtr.Zero;
        private Process _process;

        public HostControl()
        {
            InitializeComponent();
            _executablePath = Path.Combine(Environment.CurrentDirectory, "DragDropApplication.exe");
        }

        protected override void OnResize(EventArgs e)
        {
            this.Invalidate();

            User32.MoveWindow(_hostedAppHandle, 0, 0, this.Width, this.Height, true);
        }

        protected override void OnHandleCreated(EventArgs e)
        {
            try
            {
                StartProcess();
                ImportProcess();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        protected override void OnHandleDestroyed(EventArgs e)
        {
            _process.CloseMainWindow();
        }

        private void StartProcess()
        {
            ProcessStartInfo startInfo = new ProcessStartInfo(_executablePath);
            startInfo.CreateNoWindow = true;
            startInfo.WindowStyle = ProcessWindowStyle.Minimized;

            _process = Process.Start(startInfo);
            _process.EnableRaisingEvents = true;
            _process.WaitForInputIdle();
        }

        private bool ImportProcess()
        {
            WaitForHandle();

            if (_hostedAppHandle == IntPtr.Zero || this.Handle == IntPtr.Zero)
                return false;

            User32.SetWindowLong(_hostedAppHandle, (int)User32.WindowAttribute.GWL_STYLE, User32.WS_VISIBLE);

            // Resize();

            User32.SetParent(_hostedAppHandle, this.Handle);
            User32.MoveWindow(_hostedAppHandle, 0, 0, this.Width, this.Height, true);

            return true;
        }

        private void WaitForHandle()
        {
            while (true)
            {
                if (_process.MainWindowHandle != IntPtr.Zero && this.Handle != null)
                {
                    _hostedAppHandle = _process.MainWindowHandle;
                    break;
                }

                Thread.Sleep(500);
            }
        }
    }
}
